/*
 * File:   newmainXC16.c
 * Author: M.
 *
 * Created on 10 April 2017, 20:40
 * 
 * example:
 * PIC has I2C address 0x50
 * Let thing that LEDs 1H, 2H and 3H are registers
 * with addresses 0, 1 and 2 respectively
 * if external PC (BBB) write '0' -> LED is OFF, else at other
 * written value -> LED turns ON
 * if BBB Reads register 0 to 2, the PIC returns value 0 or 1 
 * LRD 2L blinks with period 2s simply for indication.
 */

#include "newmainXC16.h"

// FOR DELAYS!!!
#define FCY 62500000ULL // 1/FCY -> 1/62,5MHz = 16ns
#include <libpic30.h>
#define delay_us(x) __delay_us(x)
#define delay_ms(x) __delay_ms(x)

#include <ports.h>
#include <pps.h>


// FUNCTIONs
// ---------


// FOR SYSTEM
// ==========

void SetClock(void) {
    // Quartz = 10MHz
    // PLLPRE = 2 (default) -> 5MHz
    // PLL = 56 -> 5*56 = 280MHz
    // PLLPOST = 2 (NOT default!) -> 140MHz, divided by 2 in CPU -> result max 70MHz
    // this is minimal command time = 1/70MHz = 14.2857ns
    // BUT:
    // if use PLL = 50 (default) =>
    // 5MHz -> 250MHz -> 125MHz -> 62,5MHz in CPU. i. e. 16ns step
    PLLFBD = 48; // M=50 // 0->2; ... 48->50 (default) ... 255->(err. in datasheet?!)
    _PLLPOST = 0; // N2=2 // 0->2; 1->4 (default); 3->8
    _PLLPRE = 0; // N1=2 // 0->2 (default) do 31->33

    __builtin_write_OSCCONH(3); // Initiate Clock Switch to Primary Oscillator with PLL (NOSC=0b011)
    __builtin_write_OSCCONL(OSCCON | 1); // Switch request
    // Wait for Clock switch to occur
    while (OSCCONbits.COSC != 3);
    // Wait for PLL to lock
    while (OSCCONbits.LOCK != 1);
}

void IOinit(void) {
    // in documentation - only LATA is 0 after reset
    LATA = LATB = LATC = 0;

    // OFF Analog function (at ADCinit will be set the used)
    ANSELA = ANSELB = ANSELC = 0;
    
    // example outs
    _TRISB10 = 0;
    _TRISB11 = 0;
    _TRISB12 = 0;
    _TRISB13 = 0;
    _TRISB14 = 0;
    _TRISB15 = 0;
}

void I2C2Iinit(void) {
    I2C2ADD = 0x50; // 7-bit I2C slave address
    IEC3bits.SI2C2IE = 1; // ISR enable
    I2C2CONbits.I2CEN = 1; // Enable I2C2 module
}

int main(void) {
    SetClock();
    I2C2Iinit();
    IOinit();

    // Main loop
    while (1) {
        delay_ms(1000);
        Out2L = !Out2L;
    }

    return 0;
}

void _ISR _SI2C2Interrupt(void) {
    unsigned char tmp;
    static int flg=0; // if '1' -> registry address, else data
    static unsigned char addr=0;
    
    IFS3bits.SI2C2IF = 0; // clear flag
    
    if (!I2C2STATbits.R_W && !I2C2STATbits.D_A){
        tmp = I2C2RCV; // Address received
        flg = 1;
    } 
    else if (!I2C2STATbits.R_W && I2C2STATbits.D_A) {
        tmp = I2C2RCV;
        if (flg) addr = tmp; // Address
        else{
            // Data
            switch (addr){
                case 0: Out1H = (tmp) ? 1:0; break;
                case 1: Out2H = (tmp) ? 1:0; break;
                case 2: Out3H = (tmp) ? 1:0; break;
                default: break;
            }
        }        
        flg = !flg;         
    }
    else if (I2C2STATbits.R_W && !I2C2STATbits.D_A) {
        tmp = I2C2RCV;
        switch (addr){
            case 0: I2C2TRN = Out1H; break;
            case 1: I2C2TRN = Out2H; break;
            case 2: I2C2TRN = Out3H; break;
            default: break;
        }
        I2C2CONbits.SCLREL = 1; //Release SCL line
        while (I2C2STATbits.TBF); // while transmit
    }
}